#include <Adafruit_GFX.h>    
#include <Adafruit_ST7735.h> 
#include <SPI.h>
#include <Servo.h>

#define TFT_CS     9        
#define TFT_DC     10        
#define BUTTON_START_PIN 4   
#define BUTTON_INCREASE_PIN 2 
#define BUTTON_DECREASE_PIN 3 
#define LED_PIN 6
#define BUZZER_PIN 7  // Define the buzzer pin

Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC);
Servo myServo;  // Declare a Servo object

long countdownTime = 8 * 60 * 60;   
const long originalCountdownTime = countdownTime; 
const long interval = 1000;         
const long adjustmentTime = 30 * 60; 
const float speedFactor = 1;      

unsigned long previousMillis = 0;   
bool countdownActive = false;       
bool startButtonPressed = false;    
bool increaseButtonPressed = false; 
bool decreaseButtonPressed = false; 
bool buzzerRung = false;           // Flag to check if buzzer has rung

void updateDisplay();

void setup() {
  pinMode(BUTTON_START_PIN, INPUT_PULLUP);
  pinMode(BUTTON_INCREASE_PIN, INPUT_PULLUP);
  pinMode(BUTTON_DECREASE_PIN, INPUT_PULLUP);

  tft.initR(INITR_BLACKTAB);
  tft.setRotation(1);
  tft.fillScreen(ST7735_BLACK);
  tft.setTextSize(2);
  tft.setTextColor(ST7735_WHITE);

  myServo.attach(5);
  myServo.write(0);
  delay(500);
  pinMode(LED_PIN, OUTPUT); 
  digitalWrite(LED_PIN, HIGH);
  
  pinMode(BUZZER_PIN, OUTPUT); // Set the buzzer pin as output
  
  // Initial display of time
  updateDisplay();
}

void loop() {
  unsigned long currentMillis = millis();

  // Check start/reset button state
  if (digitalRead(BUTTON_START_PIN) == LOW) {
    if (!startButtonPressed) {
      startButtonPressed = true;
      
      if (countdownActive) {
        countdownTime = originalCountdownTime;
        countdownActive = false;
        buzzerRung = false; // Reset buzzer flag when countdown is reset
      } else {
        countdownActive = true;
      }
      
      updateDisplay();
    }
  } else {
    startButtonPressed = false;
  }

  // Check increase button state
  if (digitalRead(BUTTON_INCREASE_PIN) == LOW) {
    if (!increaseButtonPressed) {
      increaseButtonPressed = true;
      
      countdownTime += adjustmentTime;
      if (countdownTime > originalCountdownTime * 2) {
        countdownTime = originalCountdownTime * 2;
      }

      updateDisplay();
    }
  } else {
    increaseButtonPressed = false;
  }

  // Check decrease button state
  if (digitalRead(BUTTON_DECREASE_PIN) == LOW) {
    if (!decreaseButtonPressed) {
      decreaseButtonPressed = true;

      countdownTime -= adjustmentTime;
      if (countdownTime < 0) {
        countdownTime = 0;
      }

      updateDisplay();
    }
  } else {
    decreaseButtonPressed = false;
  }

  // Update countdown timer every second if active
  if (countdownActive && (currentMillis - previousMillis >= interval / speedFactor)) {
    previousMillis = currentMillis;
    if (countdownTime > 0) {
      countdownTime--;
    }
    
    long timegap = (originalCountdownTime / 6) * 1000;

    if (countdownTime % (timegap / 1000) == 0 && countdownTime <= originalCountdownTime) {
      int angle = (originalCountdownTime - countdownTime) / (timegap / 1000) * 30;
      angle = constrain(angle, 0, 180);
      myServo.write(angle);
      delay(500);
    }

    // Check if countdown has reached zero
    if (countdownTime <= 0 && !buzzerRung) {
      digitalWrite(BUZZER_PIN, HIGH);  // Turn on the buzzer
      delay(3000);                      // Wait for 3 seconds
      digitalWrite(BUZZER_PIN, LOW);   // Turn off the buzzer
      buzzerRung = true;               // Set flag to true to prevent further ringing
    }

    updateDisplay();
  }
}

void updateDisplay() {
  int hours = countdownTime / 3600;
  int minutes = (countdownTime % 3600) / 60;
  int seconds = countdownTime % 60;

  tft.fillRect(20, 60, 100, 16, ST7735_BLACK);

  tft.setCursor(20, 60);
  if (hours < 10) tft.print("0");
  tft.print(hours);
  tft.print(":");
  if (minutes < 10) tft.print("0");
  tft.print(minutes);
  tft.print(":");
  if (seconds < 10) tft.print("0");
  tft.print(seconds);
}
